<?php

declare(strict_types=1);

namespace Drupal\dp_clock_mock;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\DependencyInjection\ServiceModifierInterface;
use Drupal\Core\State\StateInterface;

/**
 * Defines a service provider for the Clock Mock module.
 */
final class DpClockMockServiceProvider implements ServiceModifierInterface {

  public const string CLOCK_MOCK_ID = 'dp_clock_mock';
  public const string TIME_MOCK_ID = 'dp_time_mock';

  /**
   * {@inheritdoc}
   */
  public function alter(ContainerBuilder $container): void {
    $state = $container->get('state');
    \assert($state instanceof StateInterface);
    // If clock mock is not set leave the clock with default implementation.
    if ($clock_mock = $state->get(self::CLOCK_MOCK_ID)) {
      $state_clock_definition = $container->getDefinition($clock_mock);
      $container->setDefinition('date_point.clock', $state_clock_definition);
      $container->setDefinition('date_point.clock.request', $state_clock_definition);
    }
    // Note that mocking this time service is pointless without mocking date
    // point clocks.
    // @see \Drupal\date_point\Time
    if ($time_mock = $state->get(self::TIME_MOCK_ID)) {
      $container->setDefinition('datetime.time', $container->getDefinition($time_mock));
    }
  }

}
